/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.fabric3.runtime.webapp.implementation.webapp;

import java.net.MalformedURLException;
import java.net.URL;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.osoa.sca.annotations.Destroy;
import org.osoa.sca.annotations.EagerInit;
import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Reference;

import org.fabric3.loader.common.LoaderContextImpl;
import org.fabric3.scdl.ComponentType;
import org.fabric3.scdl.Scope;
import org.fabric3.spi.loader.LoaderContext;
import org.fabric3.spi.loader.LoaderException;
import org.fabric3.spi.loader.LoaderRegistry;
import org.fabric3.spi.loader.LoaderUtil;
import org.fabric3.spi.loader.StAXElementLoader;

/**
 * @version $Rev: 954 $ $Date: 2007-08-31 14:25:28 -0700 (Fri, 31 Aug 2007) $
 */
@EagerInit
public class WebappLoader implements StAXElementLoader<WebappImplementation> {

    private final LoaderRegistry registry;

    public WebappLoader(@Reference LoaderRegistry registry
    ) {
        this.registry = registry;
    }

    @Init
    public void init() {
        registry.registerLoader(WebappImplementation.IMPLEMENTATION_WEBAPP, this);
    }

    @Destroy
    public void destroy() {
        registry.unregisterLoader(WebappImplementation.IMPLEMENTATION_WEBAPP);
    }

    public WebappImplementation load(XMLStreamReader reader, LoaderContext loaderContext)
            throws XMLStreamException, LoaderException {

        ComponentType componentType = loadComponentType(loaderContext);
        WebappImplementation impl = new WebappImplementation();
        impl.setComponentType(componentType);
        LoaderUtil.skipToEndElement(reader);
        return impl;
    }

    private ComponentType loadComponentType(LoaderContext context) throws LoaderException {
        URL url;
        try {
            url = new URL(context.getSourceBase(), "web.componentType");
        } catch (MalformedURLException e) {
            throw new LoaderException(e.getMessage(), e);
        }
        LoaderContext childContext = new LoaderContextImpl(context.getTargetClassLoader(), url);
        ComponentType componentType = registry.load(url, ComponentType.class, childContext);
        componentType.setImplementationScope(Scope.COMPOSITE);
        return componentType;
    }
}
